# from .report import *
from rephorm.objects.report import Report
from rephorm.objects.table_section import TableSection
from rephorm.objects.table import Table
from rephorm.objects.page_break import PageBreak
from rephorm.objects.grid import Grid
from rephorm.objects.chart import Chart
from rephorm.objects.chapter import Chapter
from .objects.text import Text
from rephorm.objects.footnote import Footnote
from rephorm.objects.chart_series import ChartSeries
from rephorm.objects.table_series import TableSeries

__version__ = "1.0.2"
__author__ = 'OGResearch team'
__credits__ = 'OGResearch'
__license__ = 'Copyright 2025 OGResearch, all rights reserved'





from rephorm.dict.legend_positions import legend_positions
from rephorm.utility.report.resolve_color import resolve_color

def update_layout(title, settings, figure, y_range = None):

    # Todo: extract this, but basically this is how to get title style in plotly for chart or for label too (I guess same way for label)
    font_style = settings.styles["chart"]["title"]["font_style"]
    if font_style == "B":
        formatted_title = f"<b>{title}</b>"
    elif font_style == "I":
        formatted_title = f"<i>{title}</i>"
    elif font_style == "BI":
        formatted_title = f"<b><i>{title}</i></b>"
    else:
        formatted_title = title

    layout = {
                "title": {
                    "text": formatted_title,
                    "yref": "container",  # stop overlaps with plot area
                    "yanchor": "top", # align to top
                    "x": 0.5,
                    "automargin": True,  # adjust margins
                    "pad": {"b": 5},  # Fine control
                    "font_size": settings.styles["chart"]["title"]["font_size"],
                    "font_family": settings.styles["chart"]["title"]["font_family"],
                    "font_color": resolve_color(settings.styles["chart"]["title"]["font_color"]),
                },
                "plot_bgcolor": resolve_color(settings.styles["chart"]["bg_color"]),
                # paper_bgcolor is kinda useless for us now,
                # because of PDF image choice it will not support transparent bg
                # https://github.com/plotly/Kaleido/issues/91
                "paper_bgcolor": 'hsla(0, 100%, 50%, 0)',
                "margin": {"l": 0,
                           "r": 0,
                           "t": 5,
                           "b": 0
                           },
                "xaxis": {
                    # TODO: Later on introduce this parameter, to set number of ticks manually.
                    # "nticks": settings.styles["chart"]["x_axis"]["ticks"]["nticks"],
                    "showgrid": settings.show_grid if settings.show_grid else True,
                    "gridcolor": resolve_color(settings.styles["chart"]["grid_color"]),
                    "mirror": settings.axis_border,
                    "linecolor": resolve_color(settings.styles["chart"]["chart_border_color"]),
                    "showline": True,
                    "ticks": "inside",
                    "tickcolor": "black",
                    "linewidth": 1,
                    "tickwidth": 1,
                    "griddash": "dot",
                    "gridwidth": 1,
                    "tickfont": {"size": settings.styles["chart"]["x_axis"]["ticks"]["font_size"],
                                 "family": settings.styles["chart"]["x_axis"]["ticks"]["font_family"],
                                 "color": settings.styles["chart"]["y_axis"]["ticks"]["font_color"]},
                    "title": {
                        "text": settings.xaxis_title,
                        "font": {
                            "size": settings.styles["chart"]["x_axis"]["label"]["font_size"],
                            "family": settings.styles["chart"]["x_axis"]["label"]["font_family"]}},
                },
                "yaxis": {
                    "zeroline": settings.zeroline if hasattr(settings, 'zeroline') else True,
                    "showgrid": settings.show_grid if hasattr(settings, 'show_grid') else True,
                    "gridcolor": resolve_color(settings.styles["chart"]["grid_color"]),
                    "mirror": settings.axis_border,
                    "linecolor": resolve_color(settings.styles["chart"]["chart_border_color"]),
                    "showline": True,
                    "ticks": "inside",
                    "tickcolor": "black",
                    "linewidth": 1,
                    "tickwidth": 1,
                    "griddash": "dot",
                    "gridwidth": 1,
                    "range": y_range,
                    "tickfont": {"size": settings.styles["chart"]["y_axis"]["ticks"]["font_size"],
                                 "family": settings.styles["chart"]["y_axis"]["ticks"]["font_family"],
                                 "color": settings.styles["chart"]["y_axis"]["ticks"]["font_color"]},
                    "title": {
                        "text": settings.yaxis_title,
                        "font": {
                            "size": settings.styles["chart"]["y_axis"]["label"]["font_size"],
                            "family": settings.styles["chart"]["y_axis"]["label"]["font_family"]}},
                },
                "yaxis2": {
                    "overlaying": "y",
                    "side": "right",
                    "linecolor": resolve_color(settings.styles["chart"]["chart_border_color"]),
                    "showline": True,
                    "ticks": "inside",
                    "tickcolor": "black",
                    "linewidth": 1,
                    "tickwidth": 1,
                    "range": y_range,
                    "tickfont": {"size": settings.styles["chart"]["y_axis2"]["ticks"]["font_size"],
                                 "family": settings.styles["chart"]["y_axis2"]["ticks"]["font_family"],
                                 "color": settings.styles["chart"]["y_axis2"]["ticks"]["font_color"]},
                    "title": {
                        "text": settings.yaxis2_title,
                        "font": {
                            "size": settings.styles["chart"]["y_axis2"]["label"]["font_size"],
                            "family": settings.styles["chart"]["y_axis2"]["label"]["font_family"]}},
                },
                "legend": {
                    "font": {
                        "size": settings.styles["chart"]["legend"]["font_size"],
                        "family": settings.styles["chart"]["legend"]["font_family"],
                        "color": resolve_color(settings.styles["chart"]["legend"]["font_color"])
                    },
                    "orientation": settings.legend_orientation.lower(),
                    "x": legend_positions.get(settings.legend_position.upper()).get("x"),
                    "y": legend_positions.get(settings.legend_position.upper()).get("y"),
                    "xanchor": legend_positions.get(settings.legend_position.upper()).get("xanchor"),
                    "yanchor": legend_positions.get(settings.legend_position.upper()).get("yanchor"),
                    "borderwidth": settings.styles["chart"]["legend"]["border_width"],
                    "yref": legend_positions.get(settings.legend_position.upper()).get("yref"),
                    "xref": legend_positions.get(settings.legend_position.upper()).get("xref"),
                    "bgcolor": resolve_color(settings.styles["chart"]["legend"]["bg_color"]),
                }
            }

    # Update conditionally only if the user explicitly specifies 'ncols'.
    legend_ncol = settings.legend_ncol
    if bool(legend_ncol):
        figure.layout.legend["entrywidth"] = 1 / legend_ncol
        figure.layout.legend["entrywidthmode"] = "fraction"

    figure.update_layout(layout)




from logging import warning

from fpdf import FPDF

from rephorm.object_mappers._globals import get_figure_map
from rephorm.utility.report.cleanup_utility import perform_cleanup
from rephorm.utility.report.layout_utility import LayoutManager
from rephorm.utility.merge.pdf_merger import overlay_pdfs
from rephorm.utility.report.report_utility import set_title_page


class ReportMapper:

    def __init__(self, pdf: FPDF):
        self.pdf = pdf

    def compile(self, report, file_name: str, cleanup:bool = False):

        rs = report.settings.styles["report"]

        self.pdf.t_margin = rs["page_margin_top"]

        # TODO: Currently, we offset all bottom spacing to the bottom margin because FPDF2
        #  automatically breaks tables as soon as it reaches the bottom margin. This prevents
        #  any overlap with the footer, since the footer is included in fpdf.b_margin.
        #  We need to apply the same approach for the top margin and top spacing
        #  (e.g., header, title padding, etc.).
        # Also, make sure to verify whether any top padding or margin we have is actually
        # padding or margin. Padding refers to inner spacing, while margin is outer spacing.
        # Follow the standard convention.

        self.pdf.b_margin = rs["page_margin_bottom"] + rs["footer"]["height"] + rs["footer"]["top_margin"]
        self.pdf.l_margin = rs["page_margin_left"]
        self.pdf.r_margin = rs["page_margin_right"]
        self.pdf.c_margin = 0

        self.pdf.add_page()
        base_file_path = f"./tmp/report_tmp.pdf"
        directory_path = "./tmp/pdf_figures"

        used_top_height = rs["header"]["height"] + rs["header"]["bottom_margin"]

        layout_manager = LayoutManager(pdf=self.pdf, used_top_height=used_top_height)

        set_title_page(self.pdf, report)

        n_children = len(report.CHILDREN)

        for i, item in enumerate(report.CHILDREN):

            x, y, w, h = layout_manager.get_default_layout()
            mapper = item._get_mapper()
            # reset width to default
            # if children does not have width set by user, use the default width, if they do:
            # check if it does not exceed pdf.w and then use children's width instead.
            if hasattr(item, "width") and item.width is not None:
                if item.width > self.pdf.w:
                    warning(
                        f"{repr(item)} width: ({item.width}) | exceeds the PDF page width: ({self.pdf.w}). Falling back to defaults"
                    )
                else:
                    w = item.width

            mapper.add_to_pdf( pdf = self.pdf, w = w, h = h, x = x, y = y)

            if i < n_children-1:
                self.pdf.add_page()

        if not get_figure_map():
            self.pdf.output(f"{file_name}.pdf")
        else:
            self.pdf.output(base_file_path)
            overlay_pdfs(base_pdf_path=base_file_path, pdf_data=get_figure_map(),
                         output_pdf_path=f"{file_name}.pdf")
            perform_cleanup(cleanup, directory_path, base_file_path)




# central storage for globals
from os.path import defpath

figure_map = []

def set_figure_map(data):
    global figure_map
    figure_map.append(data)

def get_figure_map():
    global figure_map
    return figure_map

def reset_figure_map():
    global figure_map
    figure_map = []


import fitz  # PyMuPDF

# Takes units in PT (Therefore must be converted correctly)
def overlay_pdfs(base_pdf_path, pdf_data, output_pdf_path):
    base_pdf = fitz.open(base_pdf_path)

    for pdf_info in pdf_data:
        page_num = pdf_info['page']-1
        overlay_pdf = fitz.open(pdf_info['pdf_path'])
        base_page = base_pdf[page_num]

        x = (pdf_info['x']) # X coordinate of the bottom-left corner of the overlay
        y = (pdf_info['y']) # Y coordinate of the bottom-left corner of the overlay
        width = (pdf_info['width'])  # Width of the overlay
        height = (pdf_info['height'])  # Height of the overlay

        # Define the position rectangle for the overlay
        position = fitz.Rect(x, y, x + width, y + height)

        # Draw a rectangle for visual debugging
        # red_color = (1, 0, 0)
        # base_page.draw_rect(position, color=red_color, width=1.5, fill=None)

        # Place the overlay PDF
        base_page.show_pdf_page(position, overlay_pdf, 0)
        overlay_pdf.close()

    base_pdf.save(output_pdf_path)
    print(f"Overlay PDF saved: {output_pdf_path}")